﻿#include <windows.h>

#define CM_COPY   101
#define CM_PASTE  102
#define CM_REDRAW 103
#define CM_EXIT     1
#define CM_NEW     11

LRESULT CALLBACK WndProc
(HWND hWnd, UINT Message, WPARAM wParam, LPARAM lParam);

int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance,
    LPSTR lpszCmdParam, int nCmdShow)
{
    HWND hWnd;
    WNDCLASS WndClass;
    MSG Msg;
    char szClassName[] = "Processed";
    HACCEL hAccel;

    /* заполнение класса окна */
    WndClass.style = CS_HREDRAW | CS_VREDRAW | CS_OWNDC;
    WndClass.lpfnWndProc = WndProc;
    WndClass.cbClsExtra = 0; WndClass.cbWndExtra = 0;
    WndClass.hInstance = hInstance;
    WndClass.hIcon = LoadIcon(hInstance, "ICON_1");
    WndClass.hCursor = LoadCursor(NULL, IDC_ARROW);
    WndClass.hbrBackground = (HBRUSH)GetStockObject(WHITE_BRUSH);
    WndClass.lpszMenuName = "MENU_1";
    WndClass.lpszClassName = szClassName;
    /* регистрация класса окна */
    if (!RegisterClass(&WndClass))
    {
        MessageBox(NULL, "Cannot register class", "Error", MB_OK);
        return 1;
    }
    /* создание окна */
    hWnd = CreateWindow(szClassName,
        "Metafile and Clipboard info",
        WS_OVERLAPPEDWINDOW,
        CW_USEDEFAULT, CW_USEDEFAULT,
        CW_USEDEFAULT, CW_USEDEFAULT,
        NULL, NULL, hInstance, NULL);
    if (!hWnd)
    {
        MessageBox(NULL, "Cannot create Window", "Error", MB_OK);
        return 2;
    }

    hAccel = LoadAccelerators(hInstance, "ACC_1");

    ShowWindow(hWnd, nCmdShow);
    UpdateWindow(hWnd);
    /* цикл обработки сообщений завершается по сообщению WM_QUIT */
    while (GetMessage(&Msg, // адрес струтуры сообщения
        NULL, // дескриптор окна, если NULL, то все сообщения текущей нити
        0, 0)) // диапазон сообщений (если 0,0, то все)
    {
        if (!TranslateAccelerator(hWnd, hAccel, &Msg)) // обработка акселераторов
        {
            TranslateMessage(&Msg); // трансляция виртуальных клавиш в символы
            DispatchMessage(&Msg);  // диспетчеризация сообщений
        }
    }
    return Msg.wParam; // вернуть код возврата, по сообщению WM_QUIT
}

/* ОКОННАЯ ФУНКЦИЯ */
LRESULT CALLBACK WndProc(HWND hWnd, UINT Message, WPARAM wParam, LPARAM lParam)
{
    static HMETAFILE hmf;  // дескриптор метафайла
    static HDC hdcMeta; // контекст отображения метафайла
    METAFILEPICT* metapict; // струтура для Clipboard
    HANDLE hMem; // дескриптор разделяемого блока памяти
    HDC hDC; // контекст отображения
    PAINTSTRUCT PaintStruc; // струтура для BeginPaint
    char* sz; // указатель на строчку из Clipboard
    RECT R;
    POINT point;

    switch (Message)
    {
    case WM_CREATE:
        PostMessage(hWnd, WM_COMMAND, CM_NEW, 0);
        return 0;

    case WM_PAINT:
        hDC = BeginPaint(hWnd, &PaintStruc); // контекст устройства рисования
        if (hmf) PlayMetaFile(hDC, hmf);
        EndPaint(hWnd, &PaintStruc); // закнчиваем рисование
        return 0; // возвращаем 0, если мы обрабатывали сообщение

    case WM_COMMAND:
        switch (LOWORD(wParam))
        {
        case CM_COPY:
            // выделить разделяемый блок памяти
            hMem = GlobalAlloc(GMEM_SHARE, sizeof(METAFILEPICT));
            metapict = (METAFILEPICT*)GlobalLock(hMem); // получить указатель
            metapict->mm = MM_TEXT; // система координат
            GetClientRect(hWnd, &R);  // размер изображения
            metapict->xExt = R.right;  metapict->yExt = R.bottom;
            metapict->hMF = CopyMetaFile(hmf, NULL); // сделать копию метафайла
            GlobalUnlock(hMem); // открепить блок памяти

            OpenClipboard(hWnd); // открыть Clipboard
            EmptyClipboard(); // очистить
            SetClipboardData(CF_METAFILEPICT, hMem); // поместить данные
            CloseClipboard(); // закрыть
            break;

        case CM_PASTE:
            OpenClipboard(hWnd);
            if (IsClipboardFormatAvailable(CF_TEXT)) // есть ли данные такого формата?
            {
                if (hMem = GetClipboardData(CF_TEXT))
                {
                    sz = (char*)GlobalLock(hMem); // получить указатель на строчку
                    MessageBox(hWnd, sz, "Text accepted", MB_OK);
                    GlobalUnlock(hMem);
                }
            }
            CloseClipboard(); // обязательно закрыть
            break;

        case CM_NEW:
            MessageBox(hWnd, "Нажмите правую клавишу мыши по завершению", "Начинайте рисование", MB_OK);
            if (hmf) DeleteMetaFile(hmf);  hmf = 0;
            hdcMeta = CreateMetaFile(NULL);
            InvalidateRect(hWnd, NULL, true);
            break;

        case CM_EXIT:
            SendMessage(hWnd, WM_CLOSE, 0, 0);
            break;
        }
        return 0;

    case WM_LBUTTONDOWN:
        if (!hmf)
        {
            hDC = GetDC(hWnd);
            SetPixel(hDC, LOWORD(lParam), HIWORD(lParam), RGB(255, 0, 0));
            MoveToEx(hDC, LOWORD(lParam), HIWORD(lParam), &point);
            MoveToEx(hdcMeta, LOWORD(lParam), HIWORD(lParam), &point);
            ReleaseDC(hWnd, hDC);
        }
        else MessageBox(hWnd, "Режим рисования не включен", "Ошибка рисования", MB_OK);
        return 0;

    case WM_LBUTTONUP:
        if (!hmf)
        {
            hDC = GetDC(hWnd);
            LineTo(hDC, LOWORD(lParam), HIWORD(lParam));
            LineTo(hdcMeta, LOWORD(lParam), HIWORD(lParam));
            ReleaseDC(hWnd, hDC);
        }
        return 0;

    case WM_RBUTTONDOWN:
        if (!hmf) hmf = CloseMetaFile(hdcMeta);
        InvalidateRect(hWnd, NULL, true);
        MessageBeep(MB_OK);
        return 0;

    case WM_DESTROY:
        DeleteMetaFile(hmf);
        PostQuitMessage(0);
        return 0;

    default:
        return DefWindowProc(hWnd, Message, wParam, lParam);
    }
}


